%% Project Description:
% Given the SpO2 data from 36 subjects
% Use time series features and demographic variables  to buiild a model
% to classify subjects as smokers vs Non-smokers
% (Classification problem)
clc;
clear;
close all;

%% Load data
fnames = dir("Data/*.txt"); % Select all the SpO2 data files

demgTbl = readtable('Data\participants_SpO2.csv'); % File containing participant demographic information is read into a table
%% Combine the SpO2 time series derived variables and demographic variables to obtain the features for each subject
o_p=[];
for nf = 1:numel(fnames)
    fnm = fullfile('Data',fnames(nf).name);
    data = load(fnm);
    
    row_no =strcmpi(fnames(nf).name(1:7),table2array(demgTbl(:,2)));
        
    o_p = [o_p; calculate_features(data,demgTbl(row_no,:))];
end
o_p.Properties.VariableNames = [demgTbl.Properties.VariableNames([2 3 5 6]),'Mean', 'STDev','ShanEnt','Nadir','Smoker']; % Names of features in the table
writetable(o_p, "feature_table.csv");


%%
function [output] = calculate_features(data, T)
%% Features calculated/aggregated, given a matrix of data X and demographic info. in table T

disp("Calculating Features....");
mean_acc = mean(data);
std_acc = std(data);
nadir = min(data);
shn_entpy = shannonEnt(data);

demg_feat= [T.RecordName T.Gender T.BMI T.Age];
class = ~strcmpi(T.SmokingStatus,'Non-Smoker') ; 
output = table(T.RecordName, T.Gender, T.BMI, T.Age, mean_acc, std_acc, shn_entpy, nadir,class);
    
end

function ShEnt = shannonEnt(X)
% Calculate Shannon Entropy of the data in array X
nbins= floor(log(length(X)));
pp = histcounts(X,nbins,'normalization','probability');
pp(pp==0)=[];
ShEnt =-sum(pp.*log(pp));
end