% Fisher Iris Data - Cluster analysis

clc;
close all;
clearvars;

% Load the data
load fisheriris  % Two variables: meas and species are loaded into the workspace
X = meas(:,3:4); 


% Visualize the data
figure;
plot(X(:,1),X(:,2),'k*','MarkerSize',5);
title 'Fisher''s Iris Data';
xlabel 'Petal Lengths (cm)'; 
ylabel 'Petal Widths (cm)';

% The larger cluster seems to be split into a lower variance region and a higher variance region. 
% This might indicate that the larger cluster is two, overlapping clusters.
% Cluster the data. Specify k = 3 clusters.

rng(1); % For reproducibility
[idx,C] = kmeans(X,3);  % idx is a vector of predicted cluster indices corresponding to the observations in X. C is a matrix containing the final centroid locations

% Final assignment of data to clusters
figure;
plot(X(idx==1,1),X(idx==1,2),'r.','MarkerSize',12)
hold on
plot(X(idx==2,1),X(idx==2,2),'b.','MarkerSize',12)
plot(X(idx==3,1),X(idx==3,2),'m.','MarkerSize',12)

plot(C(:,1),C(:,2),'kx',...
     'MarkerSize',15,'LineWidth',3) 
legend('Cluster 1','Cluster 2','Cluster 3','Centroids',...
       'Location','NW','AutoUpdate','off')
title 'Cluster Assignments and Centroids'
xlabel 'Petal Lengths (cm)'; 
ylabel 'Petal Widths (cm)';


% To check how the cluster assignments hold up against the true species
% labels
plot(X(strcmp(species,'setosa'),1),X(strcmp(species,'setosa'),2),'mo','MarkerSize',12)
plot(X(strcmp(species,'versicolor'),1),X(strcmp(species,'versicolor'),2),'ro','MarkerSize',12)
plot(X(strcmp(species,'virginica'),1),X(strcmp(species,'virginica'),2),'bo','MarkerSize',12)
hold off